package java.util.regex;

import java.util.Locale;
import java.util.HashMap;

enum UnicodeProp
{
    ALPHABETIC {
        @Override
        public boolean is(int codePoint) {
            return Character.isAlphabetic(codePoint);
        }
    }, 
    LETTER {
        @Override
        public boolean is(int codePoint) {
            return Character.isLetter(codePoint);
        }
    }, 
    IDEOGRAPHIC {
        @Override
        public boolean is(int codePoint) {
            return Character.isIdeographic(codePoint);
        }
    }, 
    LOWERCASE {
        @Override
        public boolean is(int codePoint) {
            return Character.isLowerCase(codePoint);
        }
    }, 
    UPPERCASE {
        @Override
        public boolean is(int codePoint) {
            return Character.isUpperCase(codePoint);
        }
    }, 
    TITLECASE {
        @Override
        public boolean is(int codePoint) {
            return Character.isTitleCase(codePoint);
        }
    }, 
    WHITE_SPACE {
        @Override
        public boolean is(int codePoint) {
            return (28672 >> Character.getType(codePoint) & 0x1) != 0x0 || (codePoint >= 9 && codePoint <= 13) || codePoint == 133;
        }
    }, 
    CONTROL {
        @Override
        public boolean is(int codePoint) {
            return Character.getType(codePoint) == 15;
        }
    }, 
    PUNCTUATION {
        @Override
        public boolean is(int codePoint) {
            return (1643118592 >> Character.getType(codePoint) & 0x1) != 0x0;
        }
    }, 
    HEX_DIGIT {
        @Override
        public boolean is(int n) {
            return UnicodeProp.10.DIGIT.is(n) || (n >= 48 && n <= 57) || (n >= 65 && n <= 70) || (n >= 97 && n <= 102) || (n >= 65296 && n <= 65305) || (n >= 65313 && n <= 65318) || (n >= 65345 && n <= 65350);
        }
    }, 
    ASSIGNED {
        @Override
        public boolean is(int codePoint) {
            return Character.getType(codePoint) != 0;
        }
    }, 
    NONCHARACTER_CODE_POINT {
        @Override
        public boolean is(int n) {
            return (n & 0xFFFE) == 0xFFFE || (n >= 64976 && n <= 65007);
        }
    }, 
    DIGIT {
        @Override
        public boolean is(int codePoint) {
            return Character.isDigit(codePoint);
        }
    }, 
    ALNUM {
        @Override
        public boolean is(int n) {
            return UnicodeProp.14.ALPHABETIC.is(n) || UnicodeProp.14.DIGIT.is(n);
        }
    }, 
    BLANK {
        @Override
        public boolean is(int codePoint) {
            return Character.getType(codePoint) == 12 || codePoint == 9;
        }
    }, 
    GRAPH {
        @Override
        public boolean is(int codePoint) {
            return (585729 >> Character.getType(codePoint) & 0x1) == 0x0;
        }
    }, 
    PRINT {
        @Override
        public boolean is(int n) {
            return (UnicodeProp.17.GRAPH.is(n) || UnicodeProp.17.BLANK.is(n)) && !UnicodeProp.17.CONTROL.is(n);
        }
    }, 
    WORD {
        @Override
        public boolean is(int codePoint) {
            return UnicodeProp.18.ALPHABETIC.is(codePoint) || (8389568 >> Character.getType(codePoint) & 0x1) != 0x0 || UnicodeProp.18.JOIN_CONTROL.is(codePoint);
        }
    }, 
    JOIN_CONTROL {
        @Override
        public boolean is(int n) {
            return n == 8204 || n == 8205;
        }
    };
    
    private static final HashMap<String, String> posix;
    private static final HashMap<String, String> aliases;
    
    public static UnicodeProp forName(String upperCase) {
        upperCase = upperCase.toUpperCase(Locale.ENGLISH);
        String s = (String)UnicodeProp.aliases.get(upperCase);
        if (s != null) {
            upperCase = s;
        }
        try {
            return valueOf(upperCase);
        }
        catch (IllegalArgumentException ex) {
            return null;
        }
    }
    
    public static UnicodeProp forPOSIXName(String s) {
        s = (String)UnicodeProp.posix.get(s.toUpperCase(Locale.ENGLISH));
        if (s == null) {
            return null;
        }
        return valueOf(s);
    }
    
    public abstract boolean is(int p0);
    
    UnicodeProp(String s, int n, UnicodeProp.1 unicodeProp) {
        this();
    }
    
    static {
        posix = new HashMap();
        aliases = new HashMap();
        UnicodeProp.posix.put("ALPHA", "ALPHABETIC");
        UnicodeProp.posix.put("LOWER", "LOWERCASE");
        UnicodeProp.posix.put("UPPER", "UPPERCASE");
        UnicodeProp.posix.put("SPACE", "WHITE_SPACE");
        UnicodeProp.posix.put("PUNCT", "PUNCTUATION");
        UnicodeProp.posix.put("XDIGIT", "HEX_DIGIT");
        UnicodeProp.posix.put("ALNUM", "ALNUM");
        UnicodeProp.posix.put("CNTRL", "CONTROL");
        UnicodeProp.posix.put("DIGIT", "DIGIT");
        UnicodeProp.posix.put("BLANK", "BLANK");
        UnicodeProp.posix.put("GRAPH", "GRAPH");
        UnicodeProp.posix.put("PRINT", "PRINT");
        UnicodeProp.aliases.put("WHITESPACE", "WHITE_SPACE");
        UnicodeProp.aliases.put("HEXDIGIT", "HEX_DIGIT");
        UnicodeProp.aliases.put("NONCHARACTERCODEPOINT", "NONCHARACTER_CODE_POINT");
        UnicodeProp.aliases.put("JOINCONTROL", "JOIN_CONTROL");
    }
}
